#!/usr/bin/perl
#Cкрипт получает данные о каналах с нескольких QTX-2, создает табличку NIT, и отсылает ее на SCR-1
#Внимание для работы требуется установленный пакет Opencaster http://www.avalpa.com/the-key-values/15-free-software/33-opencaster с библиотеками для Python!!!!

use strict;
use warnings;
use LWP::UserAgent;
use Data::Dumper;
use Encode qw(decode encode);
use DateTime;
#use JSON;

my @qtxip=('10.3.64.48','10.3.64.47'); #ip адреса QTX-4, логины и пароли устройств должны быть одинаковы.
my $qtxadmin='admin';   	#Логин для входа на QTX-4
my $qtxpasswd='password'; 	#Пароль для входа на QTX-4
my $c_version=7;		#версия таблицы NIT по умолчанию, в дальнейшем будет браться из предыдущего запуска и увеличиваться, если требуется.
my $c_network_id=39030;		#ID сети
my $netname="CactusTV";
my $br = LWP::UserAgent->new;

open (LOCK, '>> /tmp/nit_send.lock'); #блокируем выполнение скрипта, если в это время уже работает другой.
flock(LOCK, 2) or die "Не могу получить блокировку файла /tmp/nit_send.lock - $!\n";

my %CHS;
my %TSLIST;
my %TStoQAMlink;

my $count=1;

foreach my $qtx (@qtxip) #опрашиваем каждый QTX-2 из списка устройств в @qtxip и получаем данные по настройке стволов, и списку доступных каналов
{
    $br->agent('ReportsBot/1.01');
    $br->credentials( "$qtx:80", 'cgi-bin', $qtxadmin => $qtxpasswd);
    my $response;

    for (my $x=0; $x<=1; $x++)
    {
	$response = $br->get( "http://$qtx/cgi-bin/ajax.cgi?modQam=get,$x,0" );

	if ($response->is_success)
	{
	    my $text=$response->decoded_content;
	    Encode::from_to($text, "cp-1251", "utf8");
	    my @tsnum;
	    my $sr;
	    my $mode;
	    my $qmode;

	    #символьная скорость и модуляция одинаковая для всех стволов одного модулятора
	    if ($text =~/q_sr\=(\d+)/){ $sr=$1; } else { die "ОШИБКА $qtx: Не могу определить символьную скорость канала $x [$text]\n"; }
	    if ($text =~/q_mode\=(\d+)/){ $qmode=spectv_to_dvb($1); } else { die "ОШИБКА $qtx: Не могу определить символьную скорость канала $x [$text]\n"; }

	    for (my $j=1;$j<=4;$j++)
	    {
		#определяем номера стволов, проверяем на уникальность, пишем в хеш параметры стволов.
		my $tsnum;

		if ($text =~/oId$j\=(\d+)/)
		{
		    if (!exists $TSLIST{$1}{'TS'})
		    {
			$TSLIST{$1}{'CH'}=[$x,$j-1];
			$TSLIST{$1}{'SR'}=$sr;
			$TSLIST{$1}{'QM'}=$qmode;
		    
			$TStoQAMlink{$qtx}{$x}{$j-1}=$1; #привязываем к номеру канала QAM $x:$j-1 его номер для быстрого обратного поиска из полгого списка программ в дальнейшем.
		    
			$tsnum=$1;
		    }
		    else{ die "ОШИБКА $qtx: Номер ствола $1 повторяется\n"; }
		}
		else{ die "ОШИБКА $qtx: не найден номер ствола для канала $x:$j [$text]\n"; }

		#тут частота.
		if ($text =~/oF$j\=(\d+)/)
		{
		    $TSLIST{$tsnum}{'F'}=$1;
		}
		else{ die "ОШИБКА $qtx: не найдена частота ствола для канала $x:$j [$text]\n";}
	    }

	}
	else{ die $response->status_line; }
    }

    #получаем полный список всех каналов устройства
    $response = $br->get( "http://$qtx/cgi-bin/oPrg.js?oChan=128&rdm=890" );

    if ($response->is_success)
    {
	my $text=$response->decoded_content;
	Encode::from_to($text, "cp-1251", "utf8");

	#получаем все внутри скобок.
	if ( $text=~/\[(.*)\]/ ) { $text=$1; } else {die ("Ошибка!Не нашел [] в ответе!");}

	my @chlist=split('],',$text);
	foreach my $ch (@chlist)
	{
	    $ch=~ s/\'//g;
	    my ($i,$num,$ip,$qam,$prog,$sid,$name,$pmt,$st1,$st2,$st3,$st4)= split (',',$ch);
	    $qam=~ s/QAM\s*//;
	    my ($qam1,$qam2)= split (':',$qam);
	    if (!exists $CHS{$sid})
	    {
		$CHS{$sid}=[$prog,$ip,$qam1,$qam2,$name,$pmt,$st1,$st2,$st3,$st4,$qtx];	#заполняем хеш каналов
	    }
	    else
	    {
		die ("ОШИБКА! Канал с SID:$sid дублируется!");
	    }
	}
    }else { die $response->status_line; }

}

#После опроса всех устройств, бежим по полному списку каналов, заканчиваем заполение хеша стволов.
foreach my $key (sort { $a <=> $b } keys %CHS )
{
	my ($prog,$ip,$qam1,$qam2,$name,$pmt,$st1,$st2,$st3,$st4,$qtx)=@{$CHS{$key}};
	#print "$prog,$ip,$qam1,$qam2,$name,$pmt,$st1,$st2,$st3,$st4,$qtx\n";
	if (exists $TStoQAMlink{$qtx}{$qam1}{$qam2})
	{
	    my $tsnum=$TStoQAMlink{$qtx}{$qam1}{$qam2};
	    printf("%4d\t%1d:%1d\t%s" ,$key,$qam1,$qam2,$name);
	    $TSLIST{$tsnum}{'PROGS'}{$key}=[$count,$name];
	    if ($name =~ /^(\d+)\./ ) { if ($1 != $count){ print "\t\t\t==> ВНИМАНИЕ! Порядковый номер программы $count не совпадает с началом названия канала $name."; } } #если названия канала начинаются с 01. 02. и тд проверяем этот номер и номре канала по порядку
	    print "\n";
	}
	else
	{
	    die ("ОШИБКА! Не могу получить номер ствола для канала $qam1:$qam2 $qtx\n");
	}
	$count++;
}

#Так имеем список стволов, их частот и скоростей и список их каналов в хеше %TSLIST;
#Генерируем скрипт на Python для последующей генерации уже самой таблички NIT
my $NEW_NIT;

my $nit_str_begin =qq{#! /usr/bin/env python

import os

from dvbobjects.PSI.PAT import *
from dvbobjects.PSI.NIT import *

nit = network_information_section(
network_id = $c_network_id,
network_descriptor_loop = [
    network_descriptor(
	network_name = "$netname",
    ),
],
    transport_stream_loop = [
    };

$NEW_NIT= $nit_str_begin;

foreach my $key (sort { $a <=> $b } keys %TSLIST )
{
    my $freq=sprintf('0x%08d',$TSLIST{$key}{'F'}*10);
    my $mod=sprintf('0x%02d',$TSLIST{$key}{'QM'});
    my $sr=sprintf('0x%08d',$TSLIST{$key}{'SR'}*10);

    my $ts_str_begin = qq{
         transport_stream_loop_item(
         transport_stream_id = $key,
         original_network_id = $c_network_id,
         transport_descriptor_loop = [
            transport_stream_cable_descriptor(
               frequency = $freq,
               FEC_outer = 0x0000,
               modulation = $mod,
               symbol_rate = $sr,
               FEC_inner = 0x0005,
            ),};

    my $sl_str_begin = qq{
        	service_list_descriptor(
		dvb_service_descriptor_loop = [
    };

    my $chls="";
    foreach  my $pkey (sort { $a <=> $b } keys %{$TSLIST{$key}{'PROGS'}})
    {
	$chls.= qq{
		    service_descriptor_loop_item(
			service_ID = $pkey,
			service_type = 1,
		    ),
	};
    }

    my $sl_str_end = qq{
                ],
            ),
    };

    my $ts_str_end= qq{
         ],
       ),
    };

    $NEW_NIT.= $ts_str_begin.$sl_str_begin.$chls.$sl_str_end.$ts_str_end;
}

my $nit_str_end =qq{
   ],
    current_nex_indicator = 1,
    version_number = TEMP_VERSION_NIT,
    section_number = 0,
    last_section_number = 0,
)

out = open("/tmp/nit.sec", "wb")
out.write(nit.pack())
out.close
out = open("/tmp/nit.sec", "wb") # python  flush bug
out.close
    };

$NEW_NIT.=$nit_str_end;

my $script_file='/tmp/pyth_nit.py';
my $old_script='';
my $old_version_number=-1;

#Читаем старый файл скрипта в переменную для последующего сравнения, версии обоих файлов делаем одинкаковыми.
if (-e $script_file)
{
    open	(PYTH,"< /tmp/pyth_nit.py") or die "Не могу прочитать файл старого скрипта /tmp/pyth_nit.py - $1";
    binmode	(PYTH);
    {
        local $/;
        $old_script = <PYTH>;
    }

    close (PYTH);

    if ($old_script=~ /version_number\s*\=\s*(\d+),/g)
    {
	$old_version_number = $1;
	print "Предыдущая версия NIT - $old_version_number\n";
	$NEW_NIT=~ s/TEMP_VERSION_NIT/$old_version_number/g;
    }
}

if ($old_script ne $NEW_NIT)#2 скрипта получиллись разными, увеличиваем версию NIT на 1, если ее знаем
{
    if ($old_version_number!=-1){ $c_version=$old_version_number+1; };
    $NEW_NIT=~ s/version_number\s*\=\s*\w+\s*,/version_number = $c_version,/;
}

$NEW_NIT=~ s/version_number\s*\=\s*TEMP_VERSION_NIT\s*,/version_number = $c_version,/;

#Определились с версией таблицы, запускаем создание скрипта, который сгенерирует нам скрипт на python, 
#конечно идеально было бы написать весь скрипт на Python, но я его не знаю :( 
open (PYTH,"> /tmp/pyth_nit.py") or die "Не могу создать файл скрипта /tmp/pyth_nit.py - $1";
print PYTH $NEW_NIT;
close(PYTH);

#Запускаем на выполнение получившийся Python скрипт, который сгенерирует нам файл /tmp/nit.sec
print `/usr/bin/python /tmp/pyth_nit.py`;

#теперь из файла SEC сформируем файл c NIT таблицей в формате TS
print `/usr/bin/sec2ts 16 < /tmp/nit.sec > /tmp/cactus_nit.ts`;

#данные готовы к отправке, отсылаем их на SCR
print `/etc/sysconfig/utils/send_NIT_to_scr /tmp/cactus_nit.ts`;

my $filesize = -s "/tmp/cactus_nit.ts";
my $packets=$filesize/188;

print "Размер файла: $filesize байта - количество пакетов: $packets\n";
close (LOCK);
exit;

sub spectv_to_dvb
{
    my ($mode) = @_;
    # модуляции в NIT
    # 0x01=16QAM, 0x02=32QAM, 0x03=64QAM , 0x04=128QAM, 0x05=256QAM

    # модуляции в QTX
    # 0 = DOCSIS64, 1 = DOCSIS256, 2=16QAM, 3=32QAM, 4=64QAM , 5=128QAM, 6=256QAM 

    if ($mode>=2 and $mode<=6){ return $mode-1; }

    die ("ОШИБКА! Режим модуляции QTX $mode не знаю как конвертировать в DVB.");
}