#!/usr/bin/perl
########################################### Скрипт отсылает табличку NIT на SCR-1

#адрес устройства SCR-1
my $host = '10.3.64.49';
my $port = 49153;

use strict;
use warnings;
use Socket;
use Data::HexDump;

my ($ts_file) = @ARGV;

if (not defined $ts_file) { die "Укажите имя файла с таблицей NIT.\n"; }

my $ipaddr = inet_aton( $host );
my $portaddr = sockaddr_in( $port, $ipaddr);
socket( my $target, PF_INET, SOCK_DGRAM, getprotobyname('udp')) or die( "Error opening udp socket: $!");

open(NIT, '<', $ts_file) or die ("Не могу открыть файл $ts_file c таблицей NIT");
binmode (NIT);

my $packet;
my $count=0;
my %PACKETS;

while (my $len = read (NIT,$packet,188))
{
    if ($len!=188){ print "Ошибка! При чтении таблицы NIT очередной пакет короче 188 байт\n."; print HexDump $packet; exit; }

    my $sync= unpack('H*',substr($packet,0,1));
    my $type= unpack('H*',substr($packet,1,1));
    my $pid=  unpack('H*',substr($packet,2,1));

    #простые проверки, чтобы в SCR случане не закатать кривые данные.
    if ( $sync ne unpack('H*',"\x47")) 			{print "Ошибка! Первый байт пакета $count не 0x47\n."; print HexDump $packet; exit;}
    if ( $type ne unpack('H*',"\x40")  and $count==0)	{print "Ошибка! Типа первого пакета не 0x40, это не таблица NIT\n."; print HexDump $packet; exit;}
    if ( $type ne unpack('H*',"\x00")  and $count>0 )	{print "Ошибка! После превого пакета тип должен быть 0x0, это не таблица NIT\n."; exit; print HexDump $packet; exit;}
    if ( $pid  ne unpack('H*',"\x10"))			{print "Ошибка! Pid пакета на 0x10, это не таблица NIT\n."; exit; print HexDump $packet; exit;}

    $PACKETS{$count}=$packet;
    $count++;
}

#Данные загрузили слегка проверили, отсылаем на SCR-1
foreach my $key (sort { $a <=> $b } keys %PACKETS )
{
    my  $scr_header="\x71\xc0\xff\xaf".pack('C',$key+1); #Сделали заголовок отправки для SCR, длина фиксированная 5 байт + 188 = 192, в пакете это 0xС0

    my $resp;
    send($target, $scr_header.$PACKETS{$key}, 0, $portaddr) or die ("Ошибка отправки: $!");
    my $TIMEOUT=5;

    eval
    {
        local $SIG{ALRM} = sub { die "alarm time out" };
        alarm $TIMEOUT;
        recv( $target,$resp, 1500,0 )      or die "Ошибка приема данных: $!";
        alarm 0;
        1;  # return value from eval on normalcy
    } or die "Ошибка! SCR-1 не ответил в течение $TIMEOUT секунд.\n";

    if (! (unpack('H*', $resp) cmp unpack('H*', "\x02x01x03"))) #не код положительного ответа
    {
        print "Ошибка!SCR-1 вернул ошибку при запросе:\n";
        print HexDump $scr_header.$PACKETS{$key};
        print "Ответ был такой:\n";
        print HexDump $resp;
	exit;
    }
}
print "Таблица NIT успешно была загружены в SCR-1\n";
exit;
